/**
 * Copyright (C) Voltaire Ltd. 2001-2010.  ALL RIGHTS RESERVED.
 * This software product is a proprietary product of Voltaire Ltd.
 * (the "Company") and all right, title, and interest and to the software product,
 * including all associated intellectual property rights, are and shall
 * remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 * $COPYRIGHT$
 * $HEADER$
 */

#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <string.h>
#include <fca/fca_api.h>
#include <fca/config/fca_parse_specfile.h>

#include "reduce_test.h"

#define DEFAULT_MESSAGE_COUNT	0L
#define DEFAULT_PRINT_INTERVAL	50000L
#define DEFAULT_RX_QP_LEN	256
#define DEFAULT_TX_QP_LEN	64
#define DEFAULT_DEBUG_LEVEL	4
#define DEFAULT_CHILD_ID	0
#define DEFAULT_MTU		1024
#define DEFAULT_DATA		13.37
#define DEFAULT_TCP_PORT	1234
#define DEFAULT_VEC_SIZE        1
#define DEFAULT_RANK            0
#define DEFAULT_COMM_SIZE       2
#define DEFAULT_PROC_IDX        0
#define DEFAULT_NUM_PROCS       1

static void print_usage(const char *appname)
{
	printf("\n");
	printf(" Usage: reduce_test [ -s | -c <server> ] [ options ]\n");
	printf("\n");
	printf("Test options:\n");
	printf("     -s                      Run in server mode (default)\n");
	printf("     -c <server>             Run in client mode, connect to <server>\n");
	printf("     -p <tcp port>           Connection port (default: %d)\n", DEFAULT_TCP_PORT);
	printf("     -n <num>                Number of iterations, 0 is infinite (default: %ld)\n", DEFAULT_MESSAGE_COUNT);
	printf("     -i <interval>           Print interval (default: %ld)\n", DEFAULT_PRINT_INTERVAL);
	printf("     --reduce                Test Allreduce operation (default)\n");
	printf("     --barrier               Test Barrier\n");
	printf("     --bcast                 Test Bcast operation\n");
	printf("     --data <float>          Data to send (default: %.2f)\n", DEFAULT_DATA);
	printf("     --result <float>        Result to expect\n");
	printf("     --vec <size>            Data vector size (default: %d)\n", DEFAULT_VEC_SIZE);
	printf("     --ini <ini file>        Alternative INI file name\n");
	printf("\n");
	printf("Communicator options:\n");
	printf("     --rank <N>              Current rank number (Mandatory)\n");
	printf("     --comm-size <N>         Communicator size (Mandatory)\n");
	printf("     --num-procs <N>         Number of processes on the local node (default: %d)\n", DEFAULT_NUM_PROCS);
	printf("     --proc-idx <N>          Index of this process within the node (default: %d)\n", DEFAULT_PROC_IDX);
	printf("\n");
	printf("FCA options:\n");
	printf("     -d <debug level>        Debug level for FCA (0..8) (%d)\n", fca_default_config.log.level);
	printf("     --tree-flags <flags>    Communicator tree flags\n");
	printf("     --tx-len <num>          Length of send queue (%d)\n", fca_default_config.device.send_queue_len);
	printf("     --rx-len <num>          Length of receive queue (%d)\n", fca_default_config.device.recv_queue_len);
	printf("     --mtu <num>             Device MTU\n");
	printf("\n");
}

int parse_options(int argc, char **argv, char **envp, struct options *options)
{
	int i;

	memset(options, 0, sizeof(*options));
	options->fca_spec = fca_parse_spec_file(NULL);
	if (!options->fca_spec) {
		return -1;
	}

	options->message_count = DEFAULT_MESSAGE_COUNT;
	options->print_interval = DEFAULT_PRINT_INTERVAL;
	options->data = DEFAULT_DATA;
	options->vec_size = DEFAULT_VEC_SIZE;
	options->server_addr = NULL;
	options->server_port = DEFAULT_TCP_PORT;
	options->rank = -1;
	options->comm_size = -1;
	options->test_type = TEST_REDUCE;
	options->verify = 0;
	options->proc_idx = DEFAULT_PROC_IDX;
	options->num_procs = DEFAULT_NUM_PROCS;

	i = 1;
	while (i < argc && argv[i]) {
		if (!strcmp(argv[i], "-n") && argv[i + 1]) {
			options->message_count = atol(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "-i") && argv[i + 1]) {
			options->print_interval = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "-d") && argv[i + 1]) {
			options->fca_spec->config.log.level = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--rx-len") && argv[i + 1]) {
			options->fca_spec->config.device.recv_queue_len = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--tx-len") && argv[i + 1]) {
			options->fca_spec->config.device.send_queue_len = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--mtu") && argv[i + 1]) {
			options->fca_spec->config.device.mtu = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--data") && argv[i + 1]) {
			options->data = atof(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--result") && argv[i + 1]) {
			options->result = atof(argv[i + 1]);
			options->verify = 1;
			++i;
		} else if (!strcmp(argv[i], "-s")) {
			options->server_addr = NULL;
		} else if (!strcmp(argv[i], "-c") && argv[i + 1]) {
			options->server_addr = argv[i + 1];
			++i;
		} else if (!strcmp(argv[i], "--rank") && argv[i + 1]) {
			options->rank = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--comm-size") && argv[i + 1]) {
			options->comm_size = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "-p") && argv[i + 1]) {
			options->server_port = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--reduce")) {
			options->test_type = TEST_REDUCE;
			++i;
		} else if (!strcmp(argv[i], "--barrier")) {
			options->test_type = TEST_BARRIER;
			++i;
		} else if (!strcmp(argv[i], "--bcast")) {
			options->test_type = TEST_BCAST;
			++i;
		} else if (!strcmp(argv[i], "--ini") && argv[i + 1]) {
			if (options->fca_spec) {
				fca_free_init_spec(options->fca_spec);
			}
			options->fca_spec = fca_parse_spec_file(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--vec") && argv[i + 1]) {
			options->vec_size = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--tree-flags") && argv[i + 1]) {
			options->fca_spec->config.coll.tree_flags = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--proc-idx") && argv[i + 1]) {
			options->proc_idx = atoi(argv[i + 1]);
			++i;
		} else if (!strcmp(argv[i], "--num-procs") && argv[i + 1]) {
			options->num_procs = atoi(argv[i + 1]);
			++i;
		} else {
			goto error;
		}
		++i;
	}

	/* Validate options */

	if (options->rank == -1) {
		fprintf(stderr, "\nError: Rank number must be set\n");
		goto error;
	}
	if (options->comm_size == -1) {
		fprintf(stderr, "\nError: Communicator size must be set\n");
		goto error;
	}
	if (options->comm_size < 2) {
		fprintf(stderr, "\nError: Invalid communicator size (must be at least 2)\n");
		goto error;
	}
	if (options->rank < 0 || options->rank >= options->comm_size) {
		fprintf(stderr, "\nError: Invalid rank number\n");
		goto error;
	}
	if (options->proc_idx < 0 || options->proc_idx >= options->num_procs) {
		fprintf(stderr, "\nError: Invalid proc_idx\n");
		goto error;
	}

	return 0;

error:
	print_usage(argv[0]);
	return -1;
}

